/*
 * This source file is part of libRocket, the HTML/CSS Interface Middleware
 *
 * For the latest information, see http://www.librocket.com
 *
 * Copyright (c) 2008-2010 CodePoint Ltd, Shift Technology Ltd
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include <Rocket/Core/Python/Python.h>

#include "ElementGame.h"
#include <Rocket/Core/ElementDocument.h>
#include <Rocket/Core/Input.h>
#include <Rocket/Core/Factory.h>
#include <Rocket/Core/Python/ElementInstancer.h>
#include <Rocket/Core/Python/ElementWrapper.h>
#include "Defender.h"
#include "Game.h"

ElementGame::ElementGame(const Rocket::Core::String& tag) : Rocket::Core::Element(tag)
{
	game = new Game();
}

ElementGame::~ElementGame()
{		
	delete game;
}

void ElementGame::InitialisePythonInterface()
{
	PyObject* object = python::class_<ElementGame, Rocket::Core::Python::ElementWrapper<ElementGame>, python::bases<Rocket::Core::Element>, boost::noncopyable >("ElementGame", python::init<const char*>())
		.ptr();

	Rocket::Core::Factory::RegisterElementInstancer("game", new Rocket::Core::Python::ElementInstancer(object))->RemoveReference();
}

// Intercepts and handles key events.
void ElementGame::ProcessEvent(Rocket::Core::Event& event)
{
	Rocket::Core::Element::ProcessEvent(event);

	if (event == "keydown" ||
		event == "keyup")
	{
		bool key_down = event == "keydown";
		Rocket::Core::Input::KeyIdentifier key_identifier = (Rocket::Core::Input::KeyIdentifier) event.GetParameter< int >("key_identifier", 0);		

		// Process left and right keys
		if (key_down)
		{
			if (key_identifier == Rocket::Core::Input::KI_LEFT)
				game->GetDefender()->StartMove(-1.0f);
			if (key_identifier == Rocket::Core::Input::KI_RIGHT)
				game->GetDefender()->StartMove(1.0f);
			if (key_identifier == Rocket::Core::Input::KI_SPACE)
				game->GetDefender()->Fire();
		}		
		else if (!key_down)
		{
			if (key_identifier == Rocket::Core::Input::KI_LEFT)
				game->GetDefender()->StopMove(-1.0f);
			if (key_identifier == Rocket::Core::Input::KI_RIGHT)
				game->GetDefender()->StopMove(1.0f);				
		}
	}

	if (event == "load")
	{
		game->Initialise();
	}
}

// Updates the game.
void ElementGame::OnUpdate()
{
	game->Update();

	if (game->IsGameOver())
		DispatchEvent("gameover", Rocket::Core::Dictionary(), false);
}

// Renders the game.
void ElementGame::OnRender()
{
	game->Render();
}

void ElementGame::OnChildAdd(Rocket::Core::Element* element)
{
	Rocket::Core::Element::OnChildAdd(element);

	if (element == this)
		GetOwnerDocument()->AddEventListener("load", this);
}
